﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using System.ServiceModel.Discovery;
using System.ServiceModel;
using PowerClient.CalculatorProxy;
using System.Threading;

// Demo by Raffaele Rialdi. May 20, 2010
// http://www.iamraf.net
// malta@vevy.com


namespace PowerClient
{
	public sealed class ServiceDataLayer : IDisposable
	{
		#region Singleton pattern
		static ServiceDataLayer _Instance = null;
		private ServiceDataLayer()
		{
			Init();
		}

		public static ServiceDataLayer Instance
		{
			get
			{
				if(_Instance == null)
					_Instance = new ServiceDataLayer();
				return _Instance;
			}
		}

		#endregion

		private ServiceHost AnnouncementHost { get; set; }

		private void Init()
		{
			var announcementService = new AnnouncementService();
			announcementService.OnlineAnnouncementReceived += new EventHandler<AnnouncementEventArgs>(OnlineAnnouncementReceived);
			announcementService.OfflineAnnouncementReceived += new EventHandler<AnnouncementEventArgs>(OfflineAnnouncementReceived);

			AnnouncementHost = new ServiceHost(announcementService);
			AnnouncementHost.AddServiceEndpoint(new UdpAnnouncementEndpoint());
			AnnouncementHost.Open();
		}

		public void Dispose()
		{
			this.Close();
		}

		public void Close()
		{
			AnnouncementHost.Close();
		}

		public void StartNewService()
		{
			try
			{
				Process.Start(@"..\..\..\Service\bin\debug\Service.exe");
			}
			catch(Exception) { }
		}


		public event EventHandler<NewServiceEventArgs> NewServiceEvent;
		public event EventHandler<ServiceShutdownEventArgs> ServiceShutdownEvent;

		public void FindExistingServicesOfICalculator()
		{
			FindExistingServices(typeof(ICalculator));
		}

		public void FindExistingServices(Type ContractType)
		{
			ThreadPool.QueueUserWorkItem((object param) =>
				{
					DiscoveryClient client = new DiscoveryClient(new UdpDiscoveryEndpoint());
					FindCriteria criteria = new FindCriteria(ContractType);
					FindResponse response = client.Find(criteria);
					foreach(var endpoint in response.Endpoints)
					{
						ServiceInfo info = new ServiceInfo()
						{
							Address = endpoint.Address.Uri,
							ContractName = ContractType.ToString(),
						};
						if(_Instance.NewServiceEvent != null)
							_Instance.NewServiceEvent(_Instance, new NewServiceEventArgs(info));
					}
				});
		}

		static void OnlineAnnouncementReceived(object sender, AnnouncementEventArgs e)
		{
			ServiceInfo info = new ServiceInfo()
			{
				Address = e.EndpointDiscoveryMetadata.Address.Uri,
				ContractName = e.EndpointDiscoveryMetadata.ContractTypeNames.First().Name,
			};
			if(_Instance.NewServiceEvent != null)
				_Instance.NewServiceEvent(_Instance, new NewServiceEventArgs(info));
		}

		static void OfflineAnnouncementReceived(object sender, AnnouncementEventArgs e)
		{
			if(_Instance.ServiceShutdownEvent != null)
				_Instance.ServiceShutdownEvent(_Instance, new ServiceShutdownEventArgs(e.EndpointDiscoveryMetadata.Address.Uri));
		}

		public void CallService(ServiceInfo info, Operation operation)
		{
			using(CalculatorClient client = new CalculatorClient("WSHttpBinding_ICalculator", info.Address.ToString()))
			{
				switch(operation.Operator)
				{
					case Operator.Sum:
						operation.Result = client.Sum(operation.Operand1, operation.Operand2);
						break;
					case Operator.Sub:
						operation.Result = client.Sub(operation.Operand1, operation.Operand2);
						break;
					case Operator.Mul:
						operation.Result = client.Mul(operation.Operand1, operation.Operand2);
						break;
					case Operator.Div:
						operation.Result = client.Div(operation.Operand1, operation.Operand2);
						break;
				}
			}
		}
	}

	public class NewServiceEventArgs : EventArgs
	{
		public NewServiceEventArgs(ServiceInfo serviceInfo)
		{
			this.ServiceInfo = serviceInfo;
		}
		public ServiceInfo ServiceInfo { get; private set; }
	}

	public class ServiceShutdownEventArgs : EventArgs
	{
		public ServiceShutdownEventArgs(Uri uri)
		{
			this.Address = uri;
		}
		public Uri Address { get; private set; }
	}

}
